-- the following code is licensed under the creative commons CC0 license
-- basically, public domain

function createfield(cellwidth)
	local grid = {}
	for x=1,love.graphics.getWidth()/cellwidth do
		grid[x] = {}
		for y=1,love.graphics.getHeight()/cellwidth do
			grid[x][y] = false
		end
	end
	return grid
end

function fieldtoscreen(x, y)
	return (x-1)*cellwidth, (y-1)*cellwidth
end

function screentofield(x, y)
	return (x-(x%cellwidth))/cellwidth+1, (y-(y%cellwidth))/cellwidth+1
end

function getaliveneighbors(x, y)
	local cl,cr,cu,cd,out = false, false, false, false, 0
	if x > 1 then cl = true end
	if x < #field then cr = true end
	if y > 1 then cu = true end
	if y < #field[1] then cd = true end
	if cl and cu and field[x-1][y-1] then out = out + 1 end
	if cl and field[x-1][y] then out = out + 1 end
	if cl and cd and field[x-1][y+1] then out = out + 1 end
	if cu and field[x][y-1] then out = out + 1 end
	if cr and cu and field[x+1][y-1] then out = out + 1 end
	if cr and field[x+1][y] then out = out + 1 end
	if cr and cd and field[x+1][y+1] then out = out + 1 end
	if cd and field[x][y+1] then out = out + 1 end
	return out
end

function drawcells()
	love.graphics.setColor(128, 255, 000)
	for x=1,#field do
		for y=1,#field[x] do
			if field[x][y] then
				local drawx, drawy = fieldtoscreen(x, y)
				love.graphics.rectangle("fill", drawx, drawy, cellwidth, cellwidth)
			end
		end
	end
end

function drawgrid()
	love.graphics.setColor(255, 255, 255)
	love.graphics.draw(gridimage, 0, 0)
end

function drawtext()
	love.graphics.setColor(255, 255, 255)
	love.graphics.print(running and "RUNNING" or "PAUSED", 5, 17)
	love.graphics.print("Press spacebar to pause, delete to reset, and escape to exit.", 5, 35)
	love.graphics.print("Use the mount to paint cells. LMB = spawn, RMB = kill.", 5, 52)
end

function updatecells()
	for x=1,#field do
		for y=1,#field[x] do
			local alive = field[x][y]
			local aliveneighbors = getaliveneighbors(x, y)
			if alive then
				-- Any live cell with fewer than two live neighbours dies, as if caused by under-population.
				-- Any live cell with two or three live neighbours lives on to the next generation.
				-- Any live cell with more than three live neighbours dies, as if by overcrowding.
				if not (aliveneighbors == 2 or aliveneighbors == 3) then
					field[x][y] = false
				end
			else
				-- Any dead cell with exactly three live neighbours becomes a live cell, as if by reproduction.
				if aliveneighbors == 3 then
					field[x][y] = true
				end
			end
		end
	end
end

function mousecells()
	if love.mouse.isDown "l" then
		local x, y = screentofield(love.mouse.getPosition())
		field[x][y] = true
	end
	if love.mouse.isDown "r" then
		local x, y = screentofield(love.mouse.getPosition())
		field[x][y] = false
	end
end

-- callbacks
function love.load()
	running = true
	cellwidth = 5
	gridimage = love.graphics.newImage("grid.png")
	field = createfield(cellwidth)
	love.graphics.setFont(12)
	love.graphics.setCaption("Game of Life")
end

function love.draw()
	drawcells()
	drawgrid()
	drawtext()
end

local timer = 0
function love.update(dt)
	if running then
		timer = timer + dt
		if timer > 0.1 then -- fixed speed
			updatecells()
			timer = 0
		end
	end
	mousecells()
end

function love.keypressed(key)
	if key == " " then running = not running end
	if key == "escape" then love.event.push("q") end
	if key == "delete" then field = createfield(cellwidth) end
end
